;User related Stuff
!FollowMario = $00			;$00 = Center at set coordinates, $01 = Center at Mario
!CenterX = $80				;Ranges from $00 to $FF ($00 being far left and $FF being far right)
!CenterY = $70				;Ranges from $00 to $E0 ($00 being far top and $E0 being far bottom)
;Note about centerY: You can actually go up to $FF, the center will be offscreen and only the top of the circle will show.
;There's no such effect for above far top though
!InitialRadius = $00		;Ranges from $00 to $FF
!WindowType = $01			;$00 = Everything but the circle will be black, $01 = Only the circle will be black

;Can change if you want but it shouldn't be necessary
!RadiusRAM = $404000		;This holds the radius value. You can change this to anything from #$00-#$FF whenenver you want to change the radius
							;The initial value will be what's set on !InitialRadius
!CenterXRAM = $404001
!CenterYRAM = $404002
!FreeRAM = $404003			;Three bytes of free RAM. It's used to check if the current window onscreen needs to be updated
							
;Don't change
!tableRAM = $64A0

init:
LDA #!InitialRadius
STA !RadiusRAM
LDA #!CenterX
STA !CenterXRAM
LDA #!CenterY
STA !CenterYRAM
%invoke_sa1(CallCircleWindow)
JSR WindowHDMA
RTL

main:
LDA $71
BNE .levelEnded
%invoke_sa1(CallCircleWindow)
RTL

.levelEnded
LDA #$10		;\ 
TRB $6D9F		;/ Disable HDMA channel 4

LDA #$00
STA $212D
STA $212F
LDA #$17
STA $212C
STA $212E
LDA #$24
STA $40
STA $2131

STZ $2121
LDA #$00 : STA $2122	;window color low byte (SNES)
LDA #$00 : STA $2122	;window color high byte (SNES)

LDA.b #%00000000	;add in subscreen
STA $44			;also clip to black outside window
STA $2130		;(that just disables the color outside the window)
RTL

CallCircleWindow:
PHB
PHK
PLB

LDA !RadiusRAM	;\ Radius
STA $00			;/

LDA #!FollowMario
BEQ .dynamicCenter

LDA $7E	;\ Center X (relative to the left side)
CLC : ADC #$08
STA $01		;/
REP #$20
LDA $80	;\ Center Y (relative to the top of the screen)
CLC : ADC #$0010
CMP #$0100
BCC +
LDA $D3
CMP $80
BCS ++
LDA #$0000
BRA +
++
LDA #$00FF
+
STA $02		;/
SEP #$20
BRA .set

.dynamicCenter
LDA !CenterXRAM
STA $01
LDA !CenterYRAM
STA $02

.set
JSR CircleWindow
PLB
RTL

;Dynamic radius circle window
;
;Basically, takes (dx)+(dy)=r and solve for dx
;dx=sqrt(r-dy)
;Loop through all 224 scanlines and evaluate dx for each one
;And then use simetry facts to draw the circle

-
RTS
CircleWindow:
LDA !FreeRAM+1
CMP $00
BNE +
LDA !FreeRAM+2
CMP $01
BNE +
LDA !FreeRAM+3
CMP $02
BEQ -

+
STZ $2250 				;Multiplication set
LDA $00					;Evaluate radius
STA $2251				;(Needs to be done only once)
STA $2253				
STZ $2252
STZ $2254
REP #$30
LDA $2306
STA $03					;Store resulto to $03-$04
LDX #$0000				;Clear X

;X=Current scanline
.loop
TXA : LSR : SEP #$21	;A=x/2, and go to 8-bit + set carry
SBC $02					;Subtract center y
BCS +					;If result was positive, ok
EOR #$FF : INC A		;Otherwise, change signal
+						;Basicaly Math.abs(dy)
CMP $00					;Since dx=r-dy, dx, r and dy being positive integers			
BCS .clear				;dx<0 -> dy>r -> dy>r. If this happens, clear scanline

;STZ $2250 				;Multiplication remains set
STA $2251				;Evaluate dy
STA $2253
;STZ $2252				;2252 remains 00
STZ $2254
REP #$20
LDA $03					;Evaluate
SEC : SBC $2306			;r-dy (that's dx btw)
CLC						;Clear carry because sqrt counts it as an extra bit

JSL math_sqrt

;A now contains sqrt(r-dy)=dx, high byte = int part, low byte = fractional part, carry = extra 1 bit
STA $05					;And so does $05-$06, we're interested in $06 though
SEP #$21				;A = 8 bits and carry set

LDA $01					;Get center x
SBC $06					;Subtract dx (carry was already set)
BCS +					;If carry is still set, that's our initial x pos
LDA #$00				;Otherwise, just use the start of the screen
+						;Basically we're checking if $06 > $01
STA !tableRAM,x			;Set scanline initial x

LDA $01					;Repeat process
CLC : ADC $06			;But this time add dx
BCC +					;If carry is still clear, that's our final x pos
LDA #$FF				;Otherwise, use the end of the screen
+
STA !tableRAM+1,x		;Set scanline final x
REP #$20
BRA +					;Skip next bit of code

.clear
REP #$20
LDA #$00FF				;Xstart > XEnd
STA !tableRAM,x			;(That clears the scanline)
+

INX #2
CPX #$01C0
BNE .loop

LDA $00
STA !FreeRAM+1
SEP #$30
LDA $02
STA !FreeRAM+3

RTS


WindowHDMA:
;----------------------------------------------------------------------------------------------------
; Window is set to work on BG1, BG2, BG3, OBJ, Color
;----------------------------------------------------------------------------------------------------

STZ $212C		;nothing on main screen
LDA.b #%00011111	;everything on sub screen
STA $212D
LDA.b #%01100000	;backdrop color math enable
STA $40
STA $2131

;LDA.b #%00100000	;enable color window 1
STA $43
STA $2125

;Follow mario laser (blue): 3000
;Slow mario area (purpleish): 2449
;Fire laser (red): 0007

STZ $2121
LDA #$19 : STA $2122	;window color low byte (SNES)
LDA #$00 : STA $2122	;window color high byte (SNES)

LDA.b #%01000010	;add in subscreen
STA $44			;also clip to black outside window
STA $2130		;(that just disables the color outside the window)
				
REP #$20		;\
LDA #$2641		; | Use Mode 1 on register 2126
STA $4340		; | 4330 = Mode, 4331 = Register
LDA #Table	; | Address of HDMA table
STA $4342		; | 4332 = Low-Byte of table, 4333 = High-Byte of table
LDY.b #Table>>16	; | Address of HDMA table, get bank byte
STY $4344		; | 4334 = Bank-Byte of table
LDY #$40
STY $4347
SEP #$20		;/ 
LDA #$10		;\ 
TSB $6D9F		;/ Enable HDMA channel 4
RTS

Table:
db $F0 : dw $04A0
 db $F0 : dw $04A0+(112*2)
 db $00